# -*- coding: utf-8 -*-

import unittest

from jinja2.utils import generate_lorem_ipsum

from pelican.contents import Page
import pelican.settings

try:
    from . import summary
except ImportError:
    from minchin.pelican.plugins.summary import summary

# generate one paragraph, enclosed with <p>
TEST_CONTENT = str(generate_lorem_ipsum(n=1))
TEST_SUMMARY = generate_lorem_ipsum(n=1, html=False)


class TestSummary(unittest.TestCase):
    def setUp(self):
        super(TestSummary, self).setUp()
        pelican.settings.DEFAULT_CONFIG["SUMMARY_MAX_LENGTH"] = None
        pelican.settings.DEFAULT_CONFIG["SUMMARY_USE_FIRST_PARAGRAPH"] = False

        summary.register()
        summary.initialized(None)
        self.page_kwargs = {
            "content": TEST_CONTENT,
            "context": {
                "localsiteurl": "",
            },
            "metadata": {
                "summary": TEST_SUMMARY,
                "title": "foo bar",
                "author": "Blogger",
            },
        }

    def _copy_page_kwargs(self):
        # make a deep copy of page_kwargs
        page_kwargs = dict([(key, self.page_kwargs[key]) for key in self.page_kwargs])
        for key in page_kwargs:
            if not isinstance(page_kwargs[key], dict):
                break
            page_kwargs[key] = dict(
                [(subkey, page_kwargs[key][subkey]) for subkey in page_kwargs[key]]
            )

        return page_kwargs

    def test_end_summary(self):
        page_kwargs = self._copy_page_kwargs()
        del page_kwargs["metadata"]["summary"]
        page_kwargs["content"] = (
            TEST_SUMMARY + "<!-- PELICAN_END_SUMMARY -->" + TEST_CONTENT
        )
        page = Page(**page_kwargs)
        summary.extract_summary(page)
        # test both the summary and the marker removal
        self.assertEqual(page.summary, TEST_SUMMARY)
        self.assertEqual(page.content, TEST_SUMMARY + TEST_CONTENT)

    def test_begin_summary(self):
        page_kwargs = self._copy_page_kwargs()
        del page_kwargs["metadata"]["summary"]
        page_kwargs["content"] = "FOOBAR<!-- PELICAN_BEGIN_SUMMARY -->" + TEST_CONTENT
        page = Page(**page_kwargs)
        summary.extract_summary(page)
        # test both the summary and the marker removal
        self.assertEqual(page.summary, TEST_CONTENT)
        self.assertEqual(page.content, "FOOBAR" + TEST_CONTENT)

    def test_begin_end_summary(self):
        page_kwargs = self._copy_page_kwargs()
        del page_kwargs["metadata"]["summary"]
        page_kwargs["content"] = (
            "FOOBAR<!-- PELICAN_BEGIN_SUMMARY -->"
            + TEST_SUMMARY
            + "<!-- PELICAN_END_SUMMARY -->"
            + TEST_CONTENT
        )
        page = Page(**page_kwargs)
        summary.extract_summary(page)
        # test both the summary and the marker removal
        self.assertEqual(page.summary, TEST_SUMMARY)
        self.assertEqual(page.content, "FOOBAR" + TEST_SUMMARY + TEST_CONTENT)

    def test_use_first_paragraph(self):
        page_kwargs = self._copy_page_kwargs()
        del page_kwargs["metadata"]["summary"]
        pelican.settings.DEFAULT_CONFIG["SUMMARY_USE_FIRST_PARAGRAPH"] = True
        page_kwargs["content"] = "<p>" + TEST_SUMMARY + "</p>" + TEST_CONTENT
        page = Page(**page_kwargs)
        summary.extract_summary(page)
        # test both the summary and the marker removal
        self.assertEqual(page.summary, TEST_SUMMARY)
        self.assertEqual(page.content, "<p>" + TEST_SUMMARY + "</p>" + TEST_CONTENT)


if __name__ == "__main__":
    unittest.main()



"""
Summary
-------

This plugin allows easy, variable length summaries directly embedded into the
body of your articles.
"""

import logging
import re

import semantic_version

from pelican import __version__ as pelican_version
from pelican import signals
from pelican.generators import ArticlesGenerator, PagesGenerator

LOG_PREFIX = "[AutoLoader]"

logger = logging.getLogger(__name__)


def pelican_summary_as_metadata():
    """
    Determine if the installed version of Pelican stores the summary in the
    metadata table.

    In short, the Pelican version must be greater than or equal to 4.0.0.

    Pelican switched from `instance._summary` to `instance.metadata["summary"]`
    in preparation for the release of Pelican 4. (Actual commit is 06fd9b dated
    2018-02-09).

    Return:
        bool: if namespace plugins are supported
    """

    pelican_semver = semantic_version.Version(pelican_version)
    if pelican_semver.major >= 4:
        return True
    else:
        return False


def initialized(pelican):
    from pelican.settings import DEFAULT_CONFIG

    DEFAULT_CONFIG.setdefault("SUMMARY_BEGIN_MARKER", "<!-- PELICAN_BEGIN_SUMMARY -->")
    DEFAULT_CONFIG.setdefault("SUMMARY_END_MARKER", "<!-- PELICAN_END_SUMMARY -->")
    DEFAULT_CONFIG.setdefault("SUMMARY_USE_FIRST_PARAGRAPH", False)
    if pelican:
        pelican.settings.setdefault(
            "SUMMARY_BEGIN_MARKER", "<!-- PELICAN_BEGIN_SUMMARY -->"
        )
        pelican.settings.setdefault(
            "SUMMARY_END_MARKER", "<!-- PELICAN_END_SUMMARY -->"
        )
        pelican.settings.setdefault("SUMMARY_USE_FIRST_PARAGRAPH", False)

    logger.debug("%s initalized" % LOG_PREFIX)


def extract_summary(instance):
    # if summary is already specified, use it
    # if there is no content, there's nothing to do
    if (
        "summary" in instance.metadata.keys() and instance.metadata["summary"]
    ) or hasattr(instance, "_summary"):
        instance.has_summary = True
        return

    if not instance._content:
        instance.has_summary = False
        return

    begin_marker = instance.settings["SUMMARY_BEGIN_MARKER"]
    end_marker = instance.settings["SUMMARY_END_MARKER"]
    use_first_paragraph = instance.settings["SUMMARY_USE_FIRST_PARAGRAPH"]
    remove_markers = True

    content = instance._content
    begin_summary = -1
    end_summary = -1
    if begin_marker:
        begin_summary = content.find(begin_marker)
    if end_marker:
        end_summary = content.find(end_marker)

    if begin_summary == -1 and end_summary == -1 and use_first_paragraph:
        begin_marker, end_marker = "<p>", "</p>"
        remove_markers = False
        begin_summary = content.find(begin_marker)
        end_summary = content.find(end_marker)

    if begin_summary == -1 and end_summary == -1:
        instance.has_summary = False
        return

    # skip over the begin marker, if present
    if begin_summary == -1:
        begin_summary = 0
    else:
        begin_summary = begin_summary + len(begin_marker)

    if end_summary == -1:
        end_summary = None

    summary = content[begin_summary:end_summary]

    if remove_markers:
        # remove the markers from the content
        if begin_summary:
            content = content.replace(begin_marker, "", 1)
        if end_summary:
            content = content.replace(end_marker, "", 1)

    summary = re.sub(r"<div.*>", "", summary)
    summary = re.sub(r"</div>", "", summary)

    instance._content = content
    if pelican_summary_as_metadata():
        instance.metadata["summary"] = summary
    else:
        instance._summary = summary
    instance.has_summary = True


def run_plugin(generators):
    for generator in generators:
        if isinstance(generator, ArticlesGenerator):
            for article in generator.articles:
                extract_summary(article)
        elif isinstance(generator, PagesGenerator):
            for page in generator.pages:
                extract_summary(page)


def register():
    signals.initialized.connect(initialized)
    try:
        signals.all_generators_finalized.connect(run_plugin)
    except AttributeError:
        # NOTE: This results in #314 so shouldn't really be relied on
        # https://github.com/getpelican/pelican-plugins/issues/314
        signals.content_object_init.connect(extract_summary)



__title__ = "minchin.pelican.plugins.summary"
__version__ = "1.2.1"
__description__ = "This Pelican plugin allows easy, variable length summaries directly embedded into the body of your articles."
__author__ = "William Minchin"
__email__ = "w_minchin@hotmail.com"
__url__ = "https://github.com/MinchinWeb/minchin.pelican.plugins.summary"
__license__ = "GNU Affero General Public License v3"

from .summary import *



import codecs
import os
import re

import setuptools

here = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    # intentionally *not* adding an encoding option to open
    return codecs.open(os.path.join(here, *parts), "r").read()


def find_meta(*meta_file_parts, meta_key):
    """
    Extract __*meta*__ from meta_file
    """
    meta_file = read(*meta_file_parts)
    meta_match = re.search(
        r"^__{}__ = ['\"]([^'\"]*)['\"]".format(meta_key), meta_file, re.M
    )
    if meta_match:
        return meta_match.group(1)
    raise RuntimeError("Unable to find __{}__ string.".format(meta_key))


##############################################################################
#                          PACKAGE METADATA                                  #
##############################################################################
META_PATH = ["minchin", "pelican", "plugins", "summary", "__init__.py"]

# fmt: off
NAME         = find_meta(*META_PATH, meta_key="title").lower()
VERSION      = find_meta(*META_PATH, meta_key="version")
SHORT_DESC   = find_meta(*META_PATH, meta_key="description")
LONG_DESC    = read("Readme.rst")
AUTHOR       = find_meta(*META_PATH, meta_key="author")
AUTHOR_EMAIL = find_meta(*META_PATH, meta_key="email")
URL          = find_meta(*META_PATH, meta_key="url")
LICENSE      = find_meta(*META_PATH, meta_key="license")

PACKAGES     = setuptools.find_packages()
# fmt: on

INSTALL_REQUIRES = [
    "pelican",
    "semantic_version",
    "minchin.pelican.plugins.autoloader != 1.2.0",
]

EXTRA_REQUIRES = {
    "build": [
        "invoke",
        # 'pip',
        # 'setuptools>=18.0',
        "twine",
        "wheel",
        "minchin.releaser >= 0.3.1",
    ],
    "docs": [
        # 'sphinx >= 1.4',  # theme requires at least 1.4
        # 'cloud_sptheme >=1.8',
        # 'releases',
        # 'Babel >=1.3,!=2.0',  # 2.0 breaks on Windows
    ],
    "test": [
        # 'green >=1.9.4',  # v2 works
        # 'coverage',
        # 'isort',
        # 'pydocstyle',
        # 'pycodestyle',
        # 'check-manifest'
    ],
}

# full list of Classifiers at
# https://pypi.python.org/pypi?%3Aaction=list_classifiers
CLASSIFIERS = [
    #   having an unknown classifier should keep PyPI from accepting the
    #   package as an upload
    # 'Private :: Do Not Upload',
    # 'Development Status :: 1 - Planning',
    # 'Development Status :: 2 - Pre-Alpha',
    # 'Development Status :: 3 - Alpha',
    # 'Development Status :: 4 - Beta',
    "Development Status :: 5 - Production/Stable",
    # 'Development Status :: 6 - Mature',
    # 'Development Status :: 7 - Inactive',
    # 'Programming Language :: Python :: 2',
    # 'Programming Language :: Python :: 2.6',
    # 'Programming Language :: Python :: 2.7',
    # 'Programming Language :: Python :: 2 :: Only',
    "Programming Language :: Python :: 3",
    #'Programming Language :: Python :: 3.2',
    #'Programming Language :: Python :: 3.3',
    #'Programming Language :: Python :: 3.4',
    # "Programming Language :: Python :: 3.5",
    # "Programming Language :: Python :: 3.6",
    # "Programming Language :: Python :: 3.7",
    # "Programming Language :: Python :: 3.8",
    # "Programming Language :: Python :: 3.9",
    # "Programming Language :: Python :: 3.10",
    # 'Programming Language :: Python :: 3 :: Only',
    "Framework :: Pelican :: Plugins",
    "Intended Audience :: Developers",
    "Natural Language :: English",
    "Operating System :: OS Independent",
    "Topic :: Software Development :: Libraries :: Python Modules",
]
##############################################################################

if LICENSE in ["MIT License", "GNU Affero General Public License v3"]:
    CLASSIFIERS += ["License :: OSI Approved :: {}".format(LICENSE)]

# add 'all' key to EXTRA_REQUIRES
all_requires = []
for k, v in EXTRA_REQUIRES.items():
    all_requires.extend(v)
EXTRA_REQUIRES["all"] = all_requires


setuptools.setup(
    name=NAME,
    version=VERSION,
    url=URL,
    license=LICENSE,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    description=SHORT_DESC,
    long_description=LONG_DESC,
    long_description_content_type="text/x-rst",
    packages=PACKAGES,
    package_data={"": ["Readme.rst", "LICENSE.txt", "CHANGELOG.rst"]},
    include_package_data=True,
    install_requires=INSTALL_REQUIRES,
    extras_require=EXTRA_REQUIRES,
    platforms="any",
    classifiers=CLASSIFIERS,
    namespace_packages=[
        "minchin",
        "minchin.pelican",
        "minchin.pelican.plugins",
    ],
)
